#ifndef CONFIG_H_
#define CONFIG_H_

/* This file defines all the data in the data flash, and utilities to
   manage the configuration therein.  */

#define DATA_FLASH	0x00100000
#define DF_PTR(T,n)	((T *)(DATA_FLASH + 2048*(n)))
#define DF_PAGE(n)	(DATA_FLASH + 2048*(n))
#define DF_BLOCK(n)	(BLOCK_DB0 + (n))

#define P_WIRING	0
#define P_AXIS		1
#define P_LIMITS	2
#define	P_GEARING	3
#define	P_STEPS		4
#define P_HOMERPM	5
#define P_UART2		6
#define P_MOTORCAL	7
#define P_MOTORCAL2	8

/* Returns nonzero if the configuration is valid; i.e. if the board is
   configured.  This will be false after re-programming a board, and
   become true once all the configuration steps are done.  */
int config_valid (void);

/* There are 16 2k erase blocks, we put one config in each.  */
int config_wiring_done (void);
int config_axis_done (void);
int config_limits_done (void);
int config_gearing_done (void);
int config_steps_done (void);
int config_home_rpm_done (void);
int config_uart2_quadone (void);
int config_motor_calib_done (void);

#include "quad.h"

typedef struct {
  float p;
  float i;
  float imax;
  float ff;
} Pid_Data;

typedef struct {
  /* Note: LSB not used; index is port bits, masked, unshifted.  */
  int halls_to_encoder[16];
  /* encoder values when motor is driven with the given coil "on" and
     the others "off", at steady state.  */
  int encoder_at_u;
  int encoder_at_v;
  int encoder_at_w;
  /* +1 or -1 depending on winding communtation direction.  */
  int sincos_negate;
} Config_Wiring;

typedef struct {
  /* one of 'X', 'Y', 'Z' etc */
  char my_axis;
  /* if non-zero, this is the axis connected to UART2.  */
  char uart2_axis;
} Config_Axis;

typedef struct {
  /* xor pattern to make all limits active=high */
  unsigned char polarity;
  /* which limit switches we hit in the incrementing-encoder direction */
  unsigned char limit_pos;
  /* which limit switches we hit in the decrementing-encoder direction */
  unsigned char limit_neg;
  /* which switches are "home" sensors in the inc-enc dir */
  unsigned char home_pos;
  /* which switches are "home" sensors int the dec-enc dir */
  unsigned char home_neg;
  /* which switches activate limit1 output */
  unsigned char limit1;
  /* which switches activate limit2 output */
  unsigned char limit2;
  unsigned char limit1_active_low;
  unsigned char limit2_active_low;
} Config_Limits;

#define LIMITS_ACT() (LIMITS ^ config_limits.polarity)
#define PLIMIT_HIT() ((LIMITS ^ config_limits.polarity) & config_limits.limit_pos)
#define NLIMIT_HIT() ((LIMITS ^ config_limits.polarity) & config_limits.limit_neg)
#define LIMIT_HIT() ((LIMITS ^ config_limits.polarity) & (config_limits.limit_pos | config_limits.limit_neg))
#define PHOME_HIT() ((LIMITS ^ config_limits.polarity) & config_limits.home_pos)
#define NHOME_HIT() ((LIMITS ^ config_limits.polarity) & config_limits.home_neg)
#define LHOME_HIT() ((LIMITS ^ config_limits.polarity) & (config_limits.home_pos | config_limits.home_neg))
#define SLIMIT1_HIT() ((LIMITS ^ config_limits.polarity) & config_limits.limit1)
#define SLIMIT2_HIT() ((LIMITS ^ config_limits.polarity) & config_limits.limit2)

typedef struct {
  long numerator;
  long denomonator;
  /* if true, this converts 1 rotation to mm travel, else to inch
     travel. */
  int metric;
} Config_Gearing;

typedef struct {
  /* ratio of step/direction input pulses to encoder counts */
  int position_change_per_step;
} Config_Steps;

typedef struct {
  /* RPM to use to initially home the axis and locate the home/limit
     sensors.  */
  int rpm;
} Config_Home_RPM;

typedef struct {
  enum {
    U2_none,
    U2_console,
    U2_can,
    U2_mixed_can_console,
    U2_axis,
  } mode;
  int baud;
  int hw_flow_control;
} Config_UART2;

typedef struct {
  float winding_ohms;
  float winding_henries;
  float volts_per_krpm;
  float max_amps;
  float peak_amps;
  float max_volts;
  Pid_Data pos_pid;
  Pid_Data rpm_pid;
  Pid_Data current_q_pid;
  Pid_Data current_d_pid;
} Config_Motor_Calib;

#define config_wiring		(* DF_PTR (Config_Wiring, P_WIRING))
#define config_axis		(* DF_PTR (Config_Axis, P_AXIS))
#define config_limits		(* DF_PTR (Config_Limits, P_LIMITS))
#define config_gearing		(* DF_PTR (Config_Gearing, P_GEARING))
#define config_steps		(* DF_PTR (Config_Steps, P_STEPS))
#define config_home_rpm		(* DF_PTR (Config_Home_RPM, P_HOMERPM))
#define config_uart2		(* DF_PTR (Config_UART2, P_UART2))
#define config_motor_calib	(* DF_PTR (Config_Motor_Calib, P_MOTORCAL))

Config_Wiring		*config_wiring_get (void);
Config_Axis		*config_axis_get (void);
Config_Limits		*config_limits_get (void);
Config_Gearing		*config_gearing_get (void);
Config_Steps		*config_steps_get (void);
Config_Home_RPM		*config_home_rpm_get (void);
Config_UART2		*config_uart2_get (void);
Config_Motor_Calib	*config_motor_calib_get (void);

void config_wiring_save (Config_Wiring *);
void config_axis_save (Config_Axis *);
void config_limits_save (Config_Limits *);
void config_gearing_save (Config_Gearing *);
void config_steps_save (Config_Steps *);
void config_home_rpm_save (Config_Home_RPM *);
void config_uart2_save (Config_UART2 *);
void config_motor_calib_save (Config_Motor_Calib *);

#endif
