#include "iodefine.h"
#include "ivects.h"
#include "bldc.h"

/* This isn't as efficient as an interrupt-driven source; there's an
   extra "stop" bit in the transmission stream because we can't set
   the TX data register soon enough.  However, we can't set the baud
   rate for 115200 accurately enough, and the extra stop bit keeps the
   host from getting out of sync.  */

#define CONSOLE SCI1

#define RXBUFSZ 128
static char rxbuf[RXBUFSZ];
static volatile int rxput, rxget, rxcount;

#define TXBUFSZ 256
static char txbuf[TXBUFSZ];
static volatile int txput, txget, txcount;

static volatile int tx_idle = 1;

static long hz = 10000000;

static void __attribute__((interrupt))
sci1_rx_handler (void)
{
  int b;
  b = CONSOLE.RDR;
  rxbuf[rxput++] = b;
  rxput %= RXBUFSZ;
  rxcount ++;
  led_color (led_green);
}

static void __attribute__((interrupt))
sci1_tx_handler (void)
{
  int b;
  if (txcount > 0)
    {
      b = txbuf[txget++];
      CONSOLE.TDR = b;
      txget %= TXBUFSZ;
      txcount --;
      tx_idle = 0;
    }
  else
    tx_idle = 1;
}

static void
tty_init_port (int baud)
{
  int i;

    /* Disable, internal clock, div-1 */
  CONSOLE.SCR.BYTE = 0x00;

  /* async, 8N1, PCLK (50MHz) */
  CONSOLE.SMR.BYTE = 0x00;

  /* Disable smart card mode */
  CONSOLE.SCMR.BYTE = 0x00;

  /* div-8 mode in BRR */
  CONSOLE.SEMR.BYTE = 0x10;

  /* Calculate baud rate. */
  CONSOLE.BRR = (hz*4) / (16 * baud) - 1;

  for (i=2200; i; --i)
    asm("nop");

  /* Enable transmitter, receiver, tx/rx interrupts */
  CONSOLE.SCR.BYTE = 0xf0;
}

void
tty_set_hz (long new_hz)
{
  hz = new_hz;
}



void
tty_init ()
{
  MSTP(SCI1) = 0;

  /* Enable SCI1 input buffer - PD.[35] */
  PORTD.ICR.BIT.B5 = 1;
  PORTD.DDR.BIT.B3 = 1;

  rxput = rxget = rxcount = 0;

  /* SCI1 interrupts */
  _vectors[VECT(SCI1,RXI1)] = sci1_rx_handler;
  IEN(SCI1,RXI1) = 1;
  IPR(SCI1,RXI1) = 5;
  _vectors[VECT(SCI1,TXI1)] = sci1_tx_handler;
  IEN(SCI1,TXI1) = 1;
  IPR(SCI1,TXI1) = 5;

  tty_init_port (115200);
}

void
tty_putc (char c)
{
  int i;
  if (c == '\n')
    tty_putc ('\r');
#if 0
  /* TEND = 0:transmitting, 1:done */
  while (! (CONSOLE.SSR.BIT.TEND))
    ;
  CONSOLE.TDR = c;
#endif
  while (txcount >= TXBUFSZ - 5)
    asm("");
  asm("clrpsw i");
  if (tx_idle)
    {
      tx_idle = 0;
      CONSOLE.TDR = c;
    }
  else
    {
      txbuf[txput++] = c;
      txput %= TXBUFSZ;
      txcount ++;
    }
  asm("setpsw i");
}

void
tty_puts (char *s)
{
  while (*s)
    tty_putc (*s++);
}

int
write (int fd, char *buf, int len)
{
  int ilen = len;
  while (len--)
    {
      if (fd == 2)
	/*tty2_putc (*buf++)*/;
      else
	tty_putc (*buf++);
    }
  return ilen;
}

char
tty_getc ()
{
  char rv;
  while (rxcount == 0)
    ;
  rv = rxbuf[rxget++];
  rxget %= RXBUFSZ;
  rxcount --;
  if (rxcount == 0)
    led_off ();
  return rv;
}

int
tty_ready ()
{
  return rxcount;
}

int
tty_idle ()
{
  return !txcount;
}

int
tty_busy ()
{
  return txcount;
}

void
tty_discard_all ()
{
  rxput = rxget = rxcount = 0;
}
